/*
 * Unlexer.java
 * Copyright (C) 2004-2005, Bjorn Bringert (bringert@cs.chalmers.se)
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package se.chalmers.cs.gf.linearize;

import java.util.List;
import java.util.regex.Pattern;

/**
 *  Glues together strings in linearization.
 */
public class Unlexer {

        private static Unlexer instance = new Unlexer();

        public Unlexer() {}

        public String unlex(String s1, String s2) {
//                System.err.println("unlex(" + s1 + "," + s2 + ")");

                if (noSpaceAfter(s1) || noSpaceBefore(s2)) {
                        return s1 + s2;
                } else {
                        return s1 + " " + s2;
                }
        }

        private Pattern noSpaceAfterPatt = Pattern.compile("^[\\(\\-\\[]"); 
        private boolean noSpaceAfter(String s) {
                return s.length() == 0 || noSpaceAfterPatt.matcher(s).find();
        }

        private Pattern noSpaceBeforePatt = Pattern.compile("^[\\.\\,\\?\\!\\)\\:\\;\\-\\]]"); 
        private boolean noSpaceBefore(String s) {
                return s.length() == 0 || noSpaceBeforePatt.matcher(s).find();
        }

        public static Unlexer getInstance() {
                return instance;
        }

}
