/*
 * ListLanguages.java
 * Copyright (C) 2004-2005, Bjorn Bringert (bringert@cs.chalmers.se)
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package se.chalmers.cs.gf.oaa;

import se.gu.ling.trindikit.oaa.common.*;
import com.sri.oaa2.icl.*;

import se.chalmers.cs.gf.GFException;
import se.chalmers.cs.gf.abssyn.*;
import se.chalmers.cs.gf.translate.*;
import se.chalmers.cs.gf.util.Pair;
import static se.chalmers.cs.gf.util.Pair.pair;

import java.util.*;

/**
 *  Implements the list_languages solvable.
 */
public class ListLanguages extends OAASolver {

        private static String solvableString = "list_languages(Grammar, InputLangs, OutputLangs)";

        private Translators translators;

        public ListLanguages(Translators translators) {
                super(solvableString);
                this.translators = translators;
        }

        public boolean solve(IclTerm goal, IclList params, IclList answers) {
                IclTerm grammar = goal.getTerm(0);

                if (grammar.isVar()) {
                        // grammar uninstantiated
                        for (String gr : translators.listTranslators()) {
                                IclTerm ans = answer(gr, goal);
                                if (ans != null) 
                                        answers.add(ans);
                        }
                } else {
                        // grammar instantiated
                        if (!grammar.isStr())
                                return false;
                        String gr = ((IclStr)grammar).toUnquotedString();
                        IclTerm ans = answer(gr, goal);
                        if (ans != null) 
                                answers.add(ans);
                }

                return answers.size() > 0;
        }


        private IclTerm answer(String grammar, IclTerm goal) {
                Translator t = translators.getTranslator(grammar);
                if (t == null) 
                        return null;

                IclList inputLangs = stringsToIclList(t.listParsers());
                IclList outputLangs = stringsToIclList(t.listLinearizers());

                IclTerm ans = new IclStruct("list_languages", new IclStr(grammar), 
                                            inputLangs, outputLangs);
                return Unifier.getInstance().unify(goal, ans);
        }

        private IclList stringsToIclList(Collection<String> ss) {
                IclList l = new IclList();
                for (String s : ss)
                        l.add(new IclStr(s));
                return l;
        }

}
